/*
 * ExactSubtreeAutomaton.cpp
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#include "ExactSubtreeAutomaton.h"
#include <tree/ranked/PrefixRankedTree.h>
#include <automaton/PDA/InputDrivenNPDA.h>
#include <alphabet/RankedSymbol.h>

#include <alib/deque>
#include <registration/AlgoRegistration.hpp>

namespace arbology {

namespace exact {

automaton::InputDrivenNPDA < > ExactSubtreeAutomaton::construct ( const tree::PrefixRankedTree < > & tree ) {
	DefaultSymbolType S = DefaultSymbolType ( 'S' );
	automaton::InputDrivenNPDA < > res ( DefaultStateType ( 0 ), S );

	for ( const common::ranked_symbol < > & rankedSymbol : tree.getAlphabet ( ) ) {
		DefaultSymbolType symbol ( alphabet::RankedSymbol < > { rankedSymbol } );
		res.addInputSymbol ( symbol );
		res.setPushdownStoreOperation ( symbol, ext::vector < DefaultSymbolType > ( 1, S ), ext::vector < DefaultSymbolType > ( ( size_t ) rankedSymbol.getRank ( ), S ) );
	}

	int i = 1;

	for ( const common::ranked_symbol < > & rankedSymbol : tree.getContent ( ) ) {
		DefaultSymbolType symbol ( alphabet::RankedSymbol < > { rankedSymbol } );

		res.addState ( DefaultStateType ( i ) );
		res.addTransition ( DefaultStateType ( i - 1 ), symbol, DefaultStateType ( i ) );
		res.addTransition ( DefaultStateType ( 0 ), std::move ( symbol ), DefaultStateType ( i ) );
		i++;
	}

	return res;
}

auto ExactSubtreeAutomatonPrefixRankedTree = registration::AbstractRegister < ExactSubtreeAutomaton, automaton::InputDrivenNPDA < >, const tree::PrefixRankedTree < > & > ( ExactSubtreeAutomaton::construct );

} /* namespace exact */

} /* namespace arbology */
