/*
 * ExactTreePatternAutomaton.h
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#ifndef _EXACT_TREE_PATTERN_AUTOMATON_H__
#define _EXACT_TREE_PATTERN_AUTOMATON_H__

#include <alphabet/SubtreeWildcardSymbol.h>
#include <tree/ranked/PrefixRankedTree.h>
#include <automaton/PDA/InputDrivenNPDA.h>

#include <alib/deque>

namespace arbology {

namespace exact {

class ExactTreePatternAutomaton {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	template < class SymbolType, class RankType >
	static automaton::InputDrivenNPDA < common::ranked_symbol < SymbolType, RankType >, char, unsigned > construct ( const tree::PrefixRankedTree < SymbolType, RankType > & tree, const common::ranked_symbol < SymbolType, RankType > & subtreeWildcard );
};

template < class SymbolType, class RankType >
automaton::InputDrivenNPDA < common::ranked_symbol < SymbolType, RankType >, char, unsigned > ExactTreePatternAutomaton::construct ( const tree::PrefixRankedTree < SymbolType, RankType > & tree, const common::ranked_symbol < SymbolType, RankType > & subtreeWildcard ) {
	char S = alphabet::SubtreeWildcardSymbol::instance < char > ( );
	automaton::InputDrivenNPDA < common::ranked_symbol < SymbolType, RankType >, char, unsigned > res ( 0, S );

	for ( const common::ranked_symbol < SymbolType, RankType > & symbol : tree.getAlphabet ( ) ) {
		res.addInputSymbol ( symbol );
		res.setPushdownStoreOperation ( symbol, ext::vector < char > ( 1, S ), ext::vector < char > ( ( size_t ) symbol.getRank ( ), S ) );
	}

	res.addInputSymbol ( subtreeWildcard );
	res.setPushdownStoreOperation ( subtreeWildcard, ext::vector < char > ( 1, S ), ext::vector < char > { } );

	unsigned i = 1;
	ext::deque < std::pair < size_t, unsigned > > subtreeJumps;

	for ( const common::ranked_symbol < SymbolType, RankType > & symbol : tree.getContent ( ) ) {
		subtreeJumps.push_back ( std::make_pair ( ( size_t ) symbol.getRank ( ), i - 1 ) );

		res.addState ( i );
		res.addTransition ( i - 1, symbol, i );
		res.addTransition ( 0, std::move ( symbol ), i );

		while ( subtreeJumps.size ( ) && subtreeJumps.back ( ).first == 0 ) {
			res.addTransition ( subtreeJumps.back ( ).second, subtreeWildcard, i );
			subtreeJumps.pop_back ( );
		}

		if ( subtreeJumps.size ( ) ) subtreeJumps.back ( ).first--;

		i++;
	}

	return res;
}

} /* namespace exact */

} /* namespace arbology */

#endif /* _EXACT_TREE_PATTERN_AUTOMATON_H__ */
