/*
 * ToGrammarLeftRG.h
 *
 *  Created on: 8. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef TO_GRAMMAR_LEFT_RG_H_
#define TO_GRAMMAR_LEFT_RG_H_

#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightRG.h>

#include <common/createUnique.hpp>

namespace grammar {

namespace convert {

/**
 * Converts right regular grammar to left regular grammar.
 */
class ToGrammarLeftRG {
public:
	/**
	 * Performs conversion.
	 * @param grammar Right regular grammar to convert
	 * @return left regular grammar which is equivalent to source right regular grammar.
	 */
	template < class SymbolType >
	static grammar::LeftRG < SymbolType > convert(const grammar::RightRG < SymbolType > & grammar);
};

template < class SymbolType >
grammar::LeftRG < SymbolType > ToGrammarLeftRG::convert(const grammar::RightRG < SymbolType > & grammar) {
	// 1.
	SymbolType s = common::createUnique( grammar.getInitialSymbol( ), grammar.getNonterminalAlphabet(), grammar.getTerminalAlphabet() );

	grammar::LeftRG < SymbolType > lrg(s);

	for(const auto & nonterminalSymbol : grammar.getNonterminalAlphabet()) {
		lrg.addNonterminalSymbol( nonterminalSymbol );
	}

	lrg.setTerminalAlphabet( grammar.getTerminalAlphabet( ) );
	lrg.setGeneratesEpsilon( grammar.getGeneratesEpsilon( ) );

	// 2.
	for( const auto & rule : grammar.getRules( ) ) {
		const SymbolType& lhs = rule.first;

		for(const auto & ruleRHS : rule.second ) {
			if( ruleRHS.template is<ext::pair<SymbolType, SymbolType>>( ) ) {
				const ext::pair<SymbolType, SymbolType>& rhs = ruleRHS.template get<ext::pair<SymbolType, SymbolType>>();

				SymbolType leftSide = rhs.second;
				ext::pair<SymbolType, SymbolType> rightSide = ext::make_pair( lhs, rhs.first );
				lrg.addRule( leftSide, std::move ( rightSide ) );

				if( lhs == grammar.getInitialSymbol( ) ) {
					leftSide = rhs.second;
					SymbolType rightSide2 = rhs.first;
					lrg.addRule( leftSide, std::move ( rightSide2 ) );
				}
			} else {
				const SymbolType& rhs = ruleRHS.template get<SymbolType>();

				SymbolType leftSide = lrg.getInitialSymbol( );
				ext::pair<SymbolType, SymbolType> rightSide = ext::make_pair ( lhs, rhs );
				lrg.addRule( leftSide, std::move ( rightSide ) );

				if( lhs == grammar.getInitialSymbol( ) ) {
					leftSide = lrg.getInitialSymbol( );
					SymbolType rightSide2 = rhs;
					lrg.addRule( leftSide, std::move ( rightSide2 ) );
				}
			}
		}
	}

	return lrg;
}

} /* namespace convert */

} /* namespace grammar */

#endif /* TO_GRAMMAR_LEFT_RG_H_ */
