/*
 * GlushkovIndexate.cpp
 *
 *  Created on: 14. 4. 2016
 *	  Author: Tomas Pecka
 */

#include "GlushkovIndexate.h"

#include <rte/formal/FormalRTEElements.h>

#include <common/DefaultSymbolsPairType.h>

namespace rte {

// ----------------------------------------------------------------------------

common::ranked_symbol < > GlushkovIndexate::getSymbolFromGlushkovPair ( const common::ranked_symbol < > & symbol ) {
	const ext::pair < DefaultSymbolType, DefaultSymbolType > & sps = ( ( const DefaultSymbolsPairType & ) symbol.getSymbol ( ).getData ( ) );
	return common::ranked_symbol < > ( sps.first, symbol.getRank ( ) );
}

FormalRTE < > GlushkovIndexate::index ( const rte::FormalRTE < > & rte ) {
	int i = 1;

	return FormalRTE < > ( FormalRTEStructure < DefaultSymbolType, primitive::Unsigned > ( rte.getRTE ( ).getStructure ( ).accept < ext::rvalue_ref < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i ) ) );
}

// ----------------------------------------------------------------------------

ext::rvalue_ref < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTESymbolAlphabet < DefaultSymbolType, primitive::Unsigned > & node, int & i ) {
	DefaultSymbolsPairType sps = DefaultSymbolsPairType ( ext::make_pair ( DefaultSymbolType ( node.getSymbol ( ).getSymbol ( ) ), DefaultSymbolType ( i++ ) ) );
	FormalRTESymbolAlphabet < DefaultSymbolType, primitive::Unsigned > * ns = new FormalRTESymbolAlphabet < DefaultSymbolType, primitive::Unsigned > ( common::ranked_symbol < > ( DefaultSymbolType ( sps ), node.getSymbol ( ).getRank ( ) ) );

	for ( const ext::smart_ptr < const rte::FormalRTESymbol < DefaultSymbolType, primitive::Unsigned > > & e : node.getElements ( ) ) {
		ext::rvalue_ref < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > child = e->accept < ext::rvalue_ref < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i );
		ns->appendElement ( * static_cast < FormalRTESymbol < DefaultSymbolType, primitive::Unsigned > * > ( child->clone ( ) ) ); // FIXME typecast
	}

	return ext::rvalue_ref < FormalRTESymbolAlphabet < DefaultSymbolType, primitive::Unsigned > > ( ns );
}

ext::rvalue_ref < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTESymbolSubst < DefaultSymbolType, primitive::Unsigned > & node, int & ) {
	return ext::rvalue_ref < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > ( new FormalRTESymbolSubst < DefaultSymbolType, primitive::Unsigned > ( node ) );
}

ext::rvalue_ref < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTEAlternation < DefaultSymbolType, primitive::Unsigned > & node, int & i ) {
	ext::rvalue_ref < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > left = node.getLeftElement ( ).accept < ext::rvalue_ref < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i );
	ext::rvalue_ref < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > right = node.getRightElement ( ).accept < ext::rvalue_ref < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i );
	return ext::rvalue_ref < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > ( new FormalRTEAlternation < DefaultSymbolType, primitive::Unsigned > ( left, right ) );
}

ext::rvalue_ref < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTESubstitution < DefaultSymbolType, primitive::Unsigned > & node, int & i ) {
	ext::rvalue_ref < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > left = node.getLeftElement ( ).accept < ext::rvalue_ref < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i );
	ext::rvalue_ref < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > right = node.getRightElement ( ).accept < ext::rvalue_ref < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i );
	return ext::rvalue_ref < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > ( new FormalRTESubstitution < DefaultSymbolType, primitive::Unsigned > ( left, right, node.getSubstitutionSymbol ( ) ) );
}

ext::rvalue_ref < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTEIteration < DefaultSymbolType, primitive::Unsigned > & node, int & i ) {
	ext::rvalue_ref < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > element = node.getElement ( ).accept < ext::rvalue_ref < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i );
	return ext::rvalue_ref < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > ( new FormalRTEIteration < DefaultSymbolType, primitive::Unsigned > ( element, node.getSubstitutionSymbol ( ) ) );
}

ext::rvalue_ref < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTEEmpty < DefaultSymbolType, primitive::Unsigned > &, int & ) {
	return ext::rvalue_ref < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > ( new FormalRTEEmpty < DefaultSymbolType, primitive::Unsigned > ( ) );
}

// ----------------------------------------------------------------------------

} /* namespace rte */
