/*
 * HopcroftImpl.h
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#ifndef GRAPH_HOPCROFT_IMPL_H_
#define GRAPH_HOPCROFT_IMPL_H_

#include <alib/list>
#include <alib/vector>
#include <alib/unordered_map>

#include "HopcroftGraph.h"

namespace graph {

namespace isomorphism {

namespace hopcroft {

class HopcroftImpl {
public:
	explicit HopcroftImpl(Graph *g1, Graph *g2);

	bool testIsomorphism();

//private:

	// Subalgorithms
	int newInteger();
	void saveInteger();
	void restoreInteger();
	ext::vector<int> vectorPartitioning(const ext::vector<ext::vector<int>> &tuples);
	bool circleIsomorphism(ext::vector<int> a, ext::vector<int> b);
	ext::vector<int> circlePartitioning(const ext::vector<ext::vector<int>> &circles);

	// Reductions
	bool doReductions();

	// A
	bool reductionA();
	bool removeLoops();
	bool removeCorollas();
	bool removeKnots();
	bool removeDegreeOneVertices();
	bool removeSpokeCenters();
	bool removeStars();
	bool removeDumbells();

	// B
	bool reductionB();
	bool replaceClumps();
	bool replaceSkeins();

	// C
	bool reductionC();
	bool removeIsolatedLowDegreeVertices();
	bool removeIsolatedLowDegree(int degree);
	bool removeNonregularGraphsWithNoIsolatedLowDegreeVertices();
	bool removeIsolatedDegree(int degree);
	bool removeFourDegreeException();
	bool removeLowDegreeVerticesWithSolitaryFaces();
	bool removeVertexWithSolitaryFace(int sides);
	bool removeTheLastGeneralCase();
	bool removeAnotherSpecialDegreeFourCase();

	// Final
	bool testByDefinition();

	bool isRegularDegreeTwoGraph();
	bool regularDegreeTwoGraphIsomorphism();

	bool isomorphism_r(ext::vector<Vertex*> vertices1, ext::vector<Vertex*> vertices2, ext::unordered_map<Vertex*, Vertex*> &mapping);
	bool isomorphism_mapping(ext::unordered_map<Vertex*, Vertex*> &mapping);

	void dumpGraphs();
	void dumpLabels();

	Graph *m_g1;
	Graph *m_g2;
	bool m_fail;
	int m_integer;
	int m_savedInteger;
};

} // namespace hopcroft

} // namespace isomorphism

} // namespace graph

#endif // GRAPH_HOPCROFT_IMPL_H_
