/*
 * ObjectsTree.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_TREE_H_
#define OBJECTS_TREE_H_

#include <alib/tree>
#include <alib/string>
#include <sstream>

#include "ContainerBase.h"
#include "ContainerFeatures.h"

#include <object/UniqueObject.h>
#include <object/AnyObject.h>

#include <core/normalize.hpp>

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
template < class ElementType >
class ObjectsTree final : public ext::tree < ElementType >, public ContainerBase {
public:
	explicit ObjectsTree ( ext::tree < ElementType > );

	virtual ContainerBase * clone ( ) const;

	virtual ContainerBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsTree & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	virtual object::ObjectBase * inc ( ) &&;

	typedef ObjectsTree < > normalized_type;
};

template < class ElementType >
ObjectsTree < ElementType >::ObjectsTree ( ext::tree < ElementType > raw ) : ext::tree < ElementType > ( std::move ( raw ) ) {
}

template < class ElementType >
ContainerBase * ObjectsTree < ElementType >::clone ( ) const {
	return new ObjectsTree ( * this );
}

template < class ElementType >
ContainerBase * ObjectsTree < ElementType >::plunder ( ) && {
	return new ObjectsTree ( std::move ( * this ) );
}

template < class ElementType >
int ObjectsTree < ElementType >::compare ( const ObjectsTree & other ) const {
	static ext::compare < ext::tree < ElementType > > comp;

	return comp ( static_cast < const ext::tree < ElementType > & > ( * this ), static_cast < const ext::tree < ElementType > & > ( other ) );
}

template < class ElementType >
void ObjectsTree < ElementType >::operator >>( std::ostream & os ) const {
	os << "(ObjectsTree " << static_cast < const ext::tree < ElementType > & > ( * this ) << ")";
}

template < class ElementType >
ObjectsTree < ElementType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return std::move ( ss ).str ( );
}

template < class ElementType >
object::ObjectBase* ObjectsTree < ElementType >::inc() && {
	return new object::UniqueObject(object::Object(std::move(*this)), primitive::Integer(0));
}

} /* namespace container */

namespace core {

template < >
struct normalize < ext::tree < object::Object > > {
	static ext::tree < object::Object > && eval ( ext::tree < object::Object > && source ) {
		return std::move ( source );
	}
};

template < class ElementType >
struct normalize < ext::tree < ElementType > > {
	static ext::tree < object::Object > eval ( ext::tree < ElementType > && source ) {
		ext::vector < ext::tree < object::Object > > children;

		for ( ext::tree < ElementType > & child : source.getChildren ( ) ) {
			children.push_back ( normalize < ext::tree < ElementType > >::eval ( std::move ( child ) ) );
		}

		return ext::tree < object::Object > ( object::Object ( object::AnyObject < ElementType > ( std::move ( source.getData ( ) ) ) ), std::move ( children ) );
	}
};

template < class ElementType >
struct normalize < container::ObjectsTree < ElementType >, typename std::enable_if < ! std::is_same < container::ObjectsTree < ElementType >, container::ObjectsTree < > >::value >::type > {
	static container::ObjectsTree < > eval ( container::ObjectsTree < ElementType > && value ) {
		return container::ObjectsTree < > ( normalize < ext::tree < ElementType > >::eval ( std::move ( value ) ) );
	}
};

} /* namespace core */

#endif /* OBJECTS_TREE_H_ */
