#include "ComponentsTest.h"
#include <core/components.hpp>

#include <alib/set>
#include <alib/linear_set>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( ComponentsTest, "alphabet" );
CPPUNIT_TEST_SUITE_REGISTRATION ( ComponentsTest );

struct GeneralAlphabet {
};

struct NonlinearAlphabet {
};

struct SubtreeWildcard {
};

class A : public core::Components < A, ext::linear_set < std::string >, component::Set, std::tuple < GeneralAlphabet, NonlinearAlphabet >, std::string, component::Value, SubtreeWildcard > {
public:
	A ( std::string string ) : core::Components < A, ext::linear_set < std::string >, component::Set, std::tuple < GeneralAlphabet, NonlinearAlphabet >, std::string, component::Value, SubtreeWildcard > ( ext::linear_set < std::string > { string, "aaa" }, ext::linear_set < std::string > { "aaa" }, string ) {
	}
};

namespace core {

template < >
class SetConstraint< A, std::string, GeneralAlphabet > {
public:
	static bool used ( const A & a, const std::string & str ) {
		return a.accessComponent < NonlinearAlphabet > ( ).get ( ).count ( str ) || a.accessComponent < SubtreeWildcard > ( ).get ( ) == str;
	}

	static bool available ( const A &, const std::string & ) {
		return true;
	}

	static void valid ( const A &, const std::string & ) {
	}
};

template < >
class SetConstraint< A, std::string, NonlinearAlphabet > {
public:
	static bool used ( const A &, const std::string & ) {
		return false;
	}

	static bool available ( const A & a, const std::string & str ) {
		return a.accessComponent < GeneralAlphabet > ( ).get ( ).count ( str );
	}

	static void valid ( const A & a, const std::string & str ) {
		if ( a.accessComponent < SubtreeWildcard > ( ).get ( ) == str )
			throw::exception::CommonException ( "Symbol " + ( std::string ) str + "cannot be set as nonlinear variable since it is already a subtree wildcard" );
	}
};

template < >
class ElementConstraint< A, std::string, SubtreeWildcard > {
public:
	static bool available ( const A & a, const std::string & str ) {
		return a.accessComponent < GeneralAlphabet > ( ).get ( ).count ( str );
	}

	static void valid ( const A & a, const std::string & str ) {
		if ( a.accessComponent < NonlinearAlphabet > ( ).get ( ).count ( str ) )
			throw::exception::CommonException ( "Symbol " + ( std::string ) str + "cannot be set as subtree wildcard since it is already a nonlinear variable" );
	}
};

} /* namespace core */

class B : public core::Components < B, ext::set < std::string >, component::Set, GeneralAlphabet, ext::set < std::string >, component::Set, NonlinearAlphabet > {
};

namespace core {

template < >
class SetConstraint< B, std::string, GeneralAlphabet > {
public:
	static bool used ( const B &, const std::string & ) {
		return false;
	}

	static bool available ( const B &, const std::string & ) {
		return true;
	}

	static void valid ( const B &, const std::string & ) {
	}
};

template < >
class SetConstraint< B, std::string, NonlinearAlphabet > {
public:
	static bool used ( const B &, const std::string & ) {
		return false;
	}

	static bool available ( const B &, const std::string & ) {
		return true;
	}

	static void valid ( const B &, const std::string & ) {
	}
};

} /* namespace core */

void ComponentsTest::setUp ( ) {
}

void ComponentsTest::tearDown ( ) {
}

void ComponentsTest::testAdd ( ) {
	A tmp ( "2" );

	CPPUNIT_ASSERT_THROW ( tmp.accessComponent < NonlinearAlphabet > ( ).add ( "1" ), exception::CommonException );
	tmp.accessComponent < GeneralAlphabet > ( ).add ( "1" );
	tmp.accessComponent < NonlinearAlphabet > ( ).add ( "1" );

	B tmp2 = B ( );
}

void ComponentsTest::testRemove ( ) {
	A tmp ( "2" );

	tmp.accessComponent < GeneralAlphabet > ( ).add ( "1" );
	tmp.accessComponent < NonlinearAlphabet > ( ).add ( "1" );
	CPPUNIT_ASSERT_THROW ( tmp.accessComponent < GeneralAlphabet > ( ).remove ( "1" ), exception::CommonException );
	tmp.accessComponent < NonlinearAlphabet > ( ).remove ( "1" );
	tmp.accessComponent < GeneralAlphabet > ( ).remove ( "1" );
}
