/*
 * PrimitiveRegistrator.cpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#include <registry/CastRegistry.hpp>
#include <registry/NormalizeRegistry.hpp>
#include <registry/ValuePrinterRegistry.hpp>
#include <registry/ImmediateRegistry.hpp>
#include <registry/ContainerRegistry.hpp>

#include <registry/XmlComposerRegistry.hpp>
#include <registry/XmlParserRegistry.hpp>
#include <registry/XmlContainerParserRegistry.hpp>

#include <primitive/xml/Character.h>
#include <primitive/xml/Double.h>
#include <primitive/xml/Integer.h>
#include <primitive/xml/Unsigned.h>
#include <primitive/xml/Bool.h>
#include <primitive/xml/UnsignedLong.h>
#include <primitive/xml/String.h>

#include <container/xml/ObjectsVector.h>
#include <container/xml/ObjectsSet.h>
#include <container/xml/ObjectsMap.h>
#include <container/xml/ObjectsVariant.h>

#include <object/xml/AnyObject.h>

#include <common/DefaultStateType.h>
#include <common/DefaultSymbolType.h>
#include <common/DefaultRankType.h>

#include <common/ranked_symbol.hpp>
#include <alphabet/xml/RankedSymbol.h>
#include <alphabet/xml/BottomOfTheStackSymbol.h>

#include <string/xml/LinearString.h>
#include <string/xml/Epsilon.h>

namespace {

class PrimitiveRegistrator {
public:
	PrimitiveRegistrator ( ) {
		abstraction::XmlParserRegistry::registerXmlParser < ext::set < ext::pair < object::Object, object::Object > > > ( "pair_set" );

		abstraction::ContainerRegistry::registerSet < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > ( "ranked_symbol" );
		abstraction::XmlContainerParserRegistry::registerSet < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > ( "ranked_symbol" );
		abstraction::XmlParserRegistry::registerXmlParser < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > ( "ranked_symbol" );
		abstraction::XmlComposerRegistry::registerXmlComposer < ext::set < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > > ( );
		abstraction::XmlComposerRegistry::registerXmlComposer < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > ( );

		abstraction::XmlComposerRegistry::registerXmlComposer < ext::map < DefaultSymbolType, size_t > > ( );
		abstraction::XmlComposerRegistry::registerXmlComposer < ext::map < common::ranked_symbol < DefaultSymbolType, DefaultRankType >, size_t > > ( );

		abstraction::XmlParserRegistry::registerXmlParser < object::Object > ( "DefaultStateType" );

		abstraction::ContainerRegistry::registerSet < object::Object > ( "Object" );
		abstraction::XmlContainerParserRegistry::registerSet < object::Object > ( "Object" );
		abstraction::XmlParserRegistry::registerXmlParser < object::Object > ( "Object" );
		abstraction::XmlComposerRegistry::registerXmlComposer < object::Object > ( );
		abstraction::XmlComposerRegistry::registerXmlComposer < ext::set < object::Object > > ( );

		abstraction::XmlComposerRegistry::registerXmlComposer < ext::set < string::LinearString < > > > ( );

		core::xmlApi < object::Object >::template registerXmlWriter < object::AnyObject < ext::variant < alphabet::BottomOfTheStackSymbol, ext::set < common::ranked_symbol < object::Object, primitive::Unsigned > > > > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < object::AnyObject < ext::set < ext::pair < object::Object, object::Object > > > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < object::AnyObject < ext::pair < object::Object, ext::variant < string::Epsilon < object::Object >, object::Object > > > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < object::AnyObject < ext::set < object::Object > > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < object::AnyObject < unsigned int > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < container::ObjectsPair < object::Object, int > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < object::AnyObject < common::ranked_symbol < object::Object, primitive::Unsigned > > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < object::AnyObject < ext::pair < unsigned int, unsigned int > > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < object::AnyObject < ext::pair < object::Object, object::Object > > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < object::AnyObject < ext::set < char > > > ( );

	}
};

auto primitiveRegistrator = PrimitiveRegistrator ( );

} /* namespace */
