/*
 * UniqueSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "UniqueSymbol.h"
#include <sstream>
#include "Symbol.h"
#include <object/Object.h>

#include <registration/ValuePrinterRegistration.hpp>

namespace alphabet {

UniqueSymbol::UniqueSymbol(Symbol symbol, primitive::Integer id) : m_symbol(std::move(symbol)), m_id(std::move(id)) {

}

SymbolBase* UniqueSymbol::clone() const {
	return new UniqueSymbol(*this);
}

SymbolBase* UniqueSymbol::plunder() && {
	return new UniqueSymbol(std::move(*this));
}

const Symbol & UniqueSymbol::getSymbol ( ) const & {
	return m_symbol;
}

Symbol && UniqueSymbol::getSymbol ( ) && {
	return std::move ( m_symbol );
}

const primitive::Integer & UniqueSymbol::getId ( ) const & {
	return m_id;
}

primitive::Integer && UniqueSymbol::getId ( ) && {
	return std::move ( m_id );
}

int UniqueSymbol::compare(const UniqueSymbol& other) const {
	int res = m_symbol.getData().compare(other.m_symbol.getData());
	if(res == 0) res = m_id.compare(other.m_id);
	return res;
}

void UniqueSymbol::operator>>(std::ostream& out) const {
	out << "(UniqueSymbol " << m_symbol << ", " << m_id << ")";
}

UniqueSymbol::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	ss << m_symbol;
	ss << ", ";
	ss << m_id;
	ss << "}";
	return std::move(ss).str();
}

SymbolBase* UniqueSymbol::inc() && {
	std::move(m_id).inc();
	return NULL;
}

} /* namespace alphabet */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < alphabet::UniqueSymbol > ( );

} /* namespace */
