/*
 * FailStateLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "FailStateLabel.h"
#include <sax/FromXMLParserHelper.h>
#include "Label.h"
#include <object/Object.h>
#include "UniqueLabel.h"

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace label {

FailStateLabel::FailStateLabel() {

}

LabelBase* FailStateLabel::clone() const {
	return new FailStateLabel(*this);
}

LabelBase* FailStateLabel::plunder() && {
	return new FailStateLabel(std::move(*this));
}

int FailStateLabel::compare(const FailStateLabel&) const {
	return 0;
}

void FailStateLabel::operator>>(std::ostream& out) const {
	out << "(FailStateLabel)";
}

FailStateLabel::operator std::string() const {
	return FailStateLabel::instance < std::string > ( );
}

FailStateLabel FailStateLabel::parse(ext::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, FailStateLabel::getXmlTagName());
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, FailStateLabel::getXmlTagName());
	return FailStateLabel();
}

void FailStateLabel::compose(ext::deque<sax::Token>& out, const FailStateLabel & ) {
	out.emplace_back(FailStateLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(FailStateLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

LabelBase* FailStateLabel::inc() && {
	return new UniqueLabel(Label(std::move(*this)), primitive::Integer(0));
}

} /* namespace label */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < label::FailStateLabel > ( );

static auto xmlWrite = registration::XmlWriterRegister < label::FailStateLabel > ( );
static auto xmlRead = registration::XmlReaderRegister < label::FailStateLabel > ( );

static auto xmlGroup1 = registration::XmlRegisterTypeInGroup < label::Label, label::FailStateLabel > ( );
static auto xmlGroup2 = registration::XmlRegisterTypeInGroup < object::Object, label::FailStateLabel > ( );

} /* namespace */
