/*
 * HexavigesimalLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "HexavigesimalLabel.h"
#include <sax/FromXMLParserHelper.h>
#include "Label.h"
#include <object/Object.h>

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace label {

HexavigesimalLabel::HexavigesimalLabel(int hexavigesimal) : m_hexavigesimal(hexavigesimal) {

}

LabelBase* HexavigesimalLabel::clone() const {
	return new HexavigesimalLabel(*this);
}

LabelBase* HexavigesimalLabel::plunder() && {
	return new HexavigesimalLabel(std::move(*this));
}

int HexavigesimalLabel::getData() const {
	return m_hexavigesimal;
}

void HexavigesimalLabel::setData(int data) {
	this->m_hexavigesimal = data;
}

int HexavigesimalLabel::compare(const HexavigesimalLabel& other) const {
	return m_hexavigesimal - other.m_hexavigesimal;
}

void HexavigesimalLabel::operator>>(std::ostream& out) const {
	out << "(HexavigesimalLabel " << ext::toBase26(m_hexavigesimal) << ")";
}

HexavigesimalLabel::operator std::string() const {
	return ext::toBase26(m_hexavigesimal);
}

HexavigesimalLabel HexavigesimalLabel::parse(ext::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, HexavigesimalLabel::getXmlTagName());
	HexavigesimalLabel data(ext::from_string < int > (sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER)));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, HexavigesimalLabel::getXmlTagName());
	return data;
}

void HexavigesimalLabel::compose(ext::deque<sax::Token>& out, const HexavigesimalLabel & label ) {
	out.emplace_back(HexavigesimalLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(ext::to_string(label.m_hexavigesimal), sax::Token::TokenType::CHARACTER);
	out.emplace_back(HexavigesimalLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

LabelBase* HexavigesimalLabel::inc() && {
	m_hexavigesimal++;
	return NULL;
}

} /* namespace label */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < label::HexavigesimalLabel > ( );

static auto xmlWrite = registration::XmlWriterRegister < label::HexavigesimalLabel > ( );
static auto xmlRead = registration::XmlWriterRegister < label::HexavigesimalLabel > ( );

static auto xmlGroup1 = registration::XmlRegisterTypeInGroup < label::Label, label::HexavigesimalLabel > ( );
static auto xmlGroup2 = registration::XmlRegisterTypeInGroup < object::Object, label::HexavigesimalLabel > ( );

} /* namespace */
