/*
 * LabelBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef LABEL_BASE_H_
#define LABEL_BASE_H_

#include <object/ObjectBase.h>

#include <core/xmlApi.hpp>

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class LabelBase : public object::ObjectBase {
public:
	virtual LabelBase* clone() const = 0;
	virtual LabelBase* plunder() && = 0;

	virtual LabelBase* inc() && = 0;
};

} /* namespace label */

namespace core {

template < typename Type >
struct xmlApi < Type, typename std::enable_if < std::is_base_of < label::LabelBase, Type >::value >::type > {
	static Type parse ( ext::deque < sax::Token >::iterator & input ) {
		return Type::parse ( input );
	}

	static bool first ( const ext::deque < sax::Token >::const_iterator & input ) {
		return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, xmlTagName() );
	}

	static std::string xmlTagName ( ) {
		return Type::getXmlTagName();
	}

	static void compose ( ext::deque < sax::Token > & output, const Type & data ) {
		Type::compose ( output, data );
	}

};

} /* namespace core */

#endif /* LABEL_BASE_H_ */
