/*
 * ObjectLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author:Jan Travnicek 
 */

#include "ObjectLabel.h"
#include <sax/FromXMLParserHelper.h>
#include "Label.h"
#include <object/Object.h>
#include "UniqueLabel.h"

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace label {

ObjectLabel::ObjectLabel(object::Object label) : m_label(std::move(label)) {

}

LabelBase* ObjectLabel::clone() const {
	return new ObjectLabel(*this);
}

LabelBase* ObjectLabel::plunder() && {
	return new ObjectLabel(std::move(*this));
}

const object::Object& ObjectLabel::getData() const {
	return m_label;
}

int ObjectLabel::compare(const ObjectLabel& other) const {
	return m_label.getData().compare(other.m_label.getData());
}

void ObjectLabel::operator>>(std::ostream& out) const {
	out << "(ObjectLabel " << m_label << ")";
}

ObjectLabel::operator std::string() const {
	return (std::string) m_label;
}

ObjectLabel ObjectLabel::parse(ext::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, ObjectLabel::getXmlTagName());
	ObjectLabel data(core::xmlApi<object::Object>::parse(input));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, ObjectLabel::getXmlTagName());
	return data;
}

void ObjectLabel::compose(ext::deque<sax::Token>& out, const ObjectLabel & label ) {
	out.emplace_back(ObjectLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	core::xmlApi<object::Object>::compose(out, label.m_label);
	out.emplace_back(ObjectLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

LabelBase* ObjectLabel::inc() && {
	return new UniqueLabel(Label(std::move(*this)), primitive::Integer(0));
}

} /* namespace label */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < label::ObjectLabel > ( );

static auto xmlWrite = registration::XmlWriterRegister < label::ObjectLabel > ( );
static auto xmlRead = registration::XmlReaderRegister < label::ObjectLabel > ( );

static auto xmlGroup1 = registration::XmlRegisterTypeInGroup < label::Label, label::ObjectLabel > ( );
static auto xmlGroup2 = registration::XmlRegisterTypeInGroup < object::Object, label::ObjectLabel > ( );

} /* namespace */
