/*
 * UniqueLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "UniqueLabel.h"
#include <sstream>
#include <sax/FromXMLParserHelper.h>
#include "Label.h"
#include <object/Object.h>

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

#include <primitive/xml/Integer.h>

namespace label {

UniqueLabel::UniqueLabel(Label label, primitive::Integer id) : m_label(std::move(label)), m_id(std::move(id)) {

}

LabelBase* UniqueLabel::clone() const {
	return new UniqueLabel(*this);
}

LabelBase* UniqueLabel::plunder() && {
	return new UniqueLabel(std::move(*this));
}

const Label& UniqueLabel::getLabel() const {
	return m_label;
}

const primitive::Integer& UniqueLabel::getId() const {
	return m_id;
}

primitive::Integer& UniqueLabel::getId() {
	return m_id;
}

int UniqueLabel::compare(const UniqueLabel& other) const {
	int res = m_label.getData().compare(other.m_label.getData());
	if(res == 0) res = m_id.compare(other.m_id);
	return res;
}

void UniqueLabel::operator>>(std::ostream& out) const {
	out << "(UniqueLabel " << m_label << ", " << m_id << ")";
}

UniqueLabel::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	ss << m_label;
	ss << ", ";
	ss << m_id;
	ss << "}";
	return std::move(ss).str();
}

UniqueLabel UniqueLabel::parse(ext::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, UniqueLabel::getXmlTagName());

	Label firstLabel = core::xmlApi<Label>::parse(input);
	primitive::Integer secondLabel = core::xmlApi<primitive::Integer>::parse(input);

	UniqueLabel data(std::move(firstLabel), std::move(secondLabel));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, UniqueLabel::getXmlTagName());
	return data;
}

void UniqueLabel::compose(ext::deque<sax::Token>& out, const UniqueLabel & label ) {
	out.emplace_back(UniqueLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	core::xmlApi<label::Label>::compose(out, label.m_label);
	core::xmlApi<primitive::Integer>::compose(out, label.m_id);
	out.emplace_back(UniqueLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

LabelBase* UniqueLabel::inc() && {
	std::move(m_id).inc();
	return NULL;
}

} /* namespace label */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < label::UniqueLabel > ( );

static auto xmlWrite = registration::XmlWriterRegister < label::UniqueLabel > ( );
static auto xmlRead = registration::XmlReaderRegister < label::UniqueLabel > ( );

static auto xmlGroup1 = registration::XmlRegisterTypeInGroup < label::Label, label::UniqueLabel > ( );
static auto xmlGroup2 = registration::XmlRegisterTypeInGroup < object::Object, label::UniqueLabel > ( );

} /* namespace */
