/*
 * TreeAuxiliary.h
 *
 *  Created on: May 5, 2016
 *      Author: Jan Travnicek
 */

#ifndef TREE_AUXILIARY_H_
#define TREE_AUXILIARY_H_

#include <alib/set>
#include <alib/tree>
#include <alib/list>
#include <alib/algorithm>
#include <stack>
#include <common/ranked_symbol.hpp>

namespace tree {

/**
 * Parser used to get tree from XML parsed into list of Tokens.
 */
class TreeAuxiliary {
public:
	template < class SymbolType, class RankType >
	static ext::set < SymbolType > unrankSymbols ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabet );
	template < class SymbolType, class RankType >
	static ext::set < common::ranked_symbol < SymbolType, RankType > > computeBars ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabet, const SymbolType & barBase );

	template < class SymbolType, class RankType >
	static ext::tree < common::ranked_symbol < SymbolType, RankType > > toRankedTree ( const ext::tree < SymbolType > & from );
	template < class SymbolType, class RankType >
	static ext::tree < common::ranked_symbol < SymbolType, RankType > > toRankedTree (const ext::vector < common::ranked_symbol < SymbolType, RankType > > & from);
	template < class SymbolType, class RankType >
	static ext::tree < SymbolType > toUnrankedTree ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & from );

};

template < class SymbolType, class RankType >
ext::set < SymbolType > TreeAuxiliary::unrankSymbols ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabet ) {
	return ext::transform < SymbolType > ( alphabet, [&] ( const common::ranked_symbol < SymbolType, RankType > & symbol ) {
			return symbol.getSymbol ( );
	} );
}

template < class SymbolType, class RankType >
ext::set < common::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::computeBars ( const ext::set < common::ranked_symbol < SymbolType, RankType > > & alphabet, const SymbolType & barBase ) {
	return ext::transform < common::ranked_symbol < SymbolType, RankType > > ( alphabet, [&] ( const common::ranked_symbol < SymbolType, RankType > & symbol ) {
			return common::ranked_symbol < SymbolType, RankType > ( barBase, symbol.getRank ( ) );
	} );
}

template < class SymbolType, class RankType >
ext::tree < common::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::toRankedTree ( const ext::tree < SymbolType > & tree ) {
	ext::vector < ext::tree < common::ranked_symbol < SymbolType, RankType > > > children;

	for ( const ext::tree < SymbolType > & child : tree ) {
		children.push_back ( toRankedTree < SymbolType, RankType > ( child ) );
	}

	unsigned size = children.size ( );
	return ext::tree < common::ranked_symbol < SymbolType, RankType > > ( common::ranked_symbol < SymbolType, RankType > ( tree.getData ( ), size ), std::move ( children ) );
}

template < class SymbolType, class RankType >
ext::tree < common::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::toRankedTree ( const ext::vector < common::ranked_symbol < SymbolType, RankType > > & from ) {

	std::stack < ext::tree < common::ranked_symbol < SymbolType, RankType > > > tree_stack;

	for ( auto x : from ) {
		if ( ( unsigned ) x.getRank ( ) == 0 ) {
			tree_stack.push ( ext::tree < common::ranked_symbol < SymbolType, RankType > > ( common::ranked_symbol < SymbolType, RankType > ( x.getSymbol ( ), x.getRank ( ) ), ext::vector < ext::tree < common::ranked_symbol < SymbolType, RankType > > > ( ) ) );
		} else {
			unsigned children_size = ( unsigned ) x.getRank ( );
			ext::list < ext::tree < common::ranked_symbol < SymbolType, RankType > > > childrenList;

			for ( unsigned i = 0; i < children_size; ++i ) {
				ext::tree < common::ranked_symbol < SymbolType, RankType > > child = tree_stack.top ( );
				tree_stack.pop ( );
				childrenList.push_front ( child );
			}

			ext::vector < ext::tree < common::ranked_symbol < SymbolType, RankType > > > children {
				std::begin ( childrenList ), std::end ( childrenList )
			};
			tree_stack.push ( ext::tree < common::ranked_symbol < SymbolType, RankType > > ( common::ranked_symbol < SymbolType, RankType > ( x.getSymbol ( ), x.getRank ( ) ), std::move ( children ) ) );
		}
	}

	return tree_stack.top ( );
}


template < class SymbolType, class RankType >
ext::tree < SymbolType > TreeAuxiliary::toUnrankedTree ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree ) {
	ext::vector < ext::tree < SymbolType > > children;

	for ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & child : tree ) {
		children.push_back ( toUnrankedTree < SymbolType, RankType > ( child ) );
	}

	return ext::tree < SymbolType > ( tree.getData ( ).getSymbol ( ), std::move ( children ) );
}

} /* namespace tree */

#endif /* TREE_AUXILIARY_H_ */
