#include "SparseBoolVectorTest.h"

#include <common/SparseBoolVector.hpp>

#include <factory/XmlDataFactory.hpp>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( SparseBoolVectorTest, "common" );
CPPUNIT_TEST_SUITE_REGISTRATION ( SparseBoolVectorTest );

void SparseBoolVectorTest::setUp ( ) {
}

void SparseBoolVectorTest::tearDown ( ) {
}

void SparseBoolVectorTest::testVectorCompatibility() {
	ext::vector < bool > orig;
	for ( int i = 0; i < 1000; ++i ) {
		orig.push_back ( false );
	}
	for ( int i = 0; i < 20; ++i ) {
		orig.push_back ( true );
		orig.push_back ( true );
		orig.push_back ( false );
		orig.push_back ( true );
		orig.push_back ( true );
		orig.push_back ( false );
		orig.push_back ( true );
		orig.push_back ( false );
		orig.push_back ( false );
		orig.push_back ( true );
	}
	for ( int i = 0; i < 1000; ++i ) {
		orig.push_back ( false );
	}
	orig.push_back ( true );
	orig.push_back ( true );
	orig.push_back ( false );
	orig.push_back ( true );
	orig.push_back ( true );
	orig.push_back ( false );
	orig.push_back ( true );
	orig.push_back ( false );
	orig.push_back ( false );
	orig.push_back ( true );

	common::SparseBoolVector compressed ( orig );
	ext::vector < bool > copy ( compressed );

/*	std::cout << orig << std::endl;
	std::cout << copy << std::endl;
	std::cout << compressed.data ( ) << std::endl;*/

	CPPUNIT_ASSERT ( orig == copy );
}

ext::vector < bool > createTestVectorBool ( size_t size ) {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	ext::vector < bool > ref;
	while ( ref.size ( ) < size ) {
		for ( size_t i = 0; i < sizeof ( unsigned long long ) * 8; ++i ) {
			ref.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
		}
		for ( size_t i = 0; i < 100; ++i ) {
			ref.push_back ( false );
		}
	}
	ref.resize ( size );
	return ref;
}

void testOffset2 ( size_t size, size_t shift ) {
	ext::vector < bool > ref = createTestVectorBool ( size );

	common::SparseBoolVector data = ref;
	data <<= shift;
	ref <<= shift;

/*	std::cout << "size = " << size << std::endl;
	std::cout << "shift = " << shift << std::endl;
	std::cout << "data = " << data.data ( ) << std::endl;
	std::cout << "ref = " << common::SparseBoolVector ( ref ).data ( ) << std::endl;*/

	CPPUNIT_ASSERT ( data == common::SparseBoolVector ( ref ) );
}

void SparseBoolVectorTest::testShifts4() {
	for ( size_t shift = 0 ; shift < 100; ++ shift ) {
		testOffset2 ( 0, shift );
		testOffset2 ( 31, shift );
		testOffset2 ( 32, shift );
		testOffset2 ( 33, shift );
		testOffset2 ( 63, shift );
		testOffset2 ( 64, shift );
		testOffset2 ( 65, shift );
		testOffset2 ( 1023, shift );
		testOffset2 ( 1024, shift );
		testOffset2 ( 1025, shift );
		testOffset2 ( 2000, shift );

	}
}

void SparseBoolVectorTest::testShifts5() {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	common::SparseBoolVector data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	common::SparseBoolVector data2 = data;
	data2 <<= 64;

	data <<= 32;
	data <<= 16;
	data <<= 8;
	data <<= 4;
	data <<= 2;
	data <<= 1;
	data <<= 1;

/*	std::cout << "data  = " << data.data ( )  << std::endl;
	std::cout << "data2 = " << data2.data ( ) << std::endl;*/

	data2 <<= 1;
	data2 <<= 2;
	data2 <<= 4;
	data2 <<= 8;
	data2 <<= 16;
	data2 <<= 32;
	data2 <<= 1;

	data <<= 27;
	data <<= 17;
	data <<= 13;
	data <<= 7;

/*	std::cout << "data  = " << data.data ( )  << std::endl;
	std::cout << "data2 = " << data2.data ( ) << std::endl;*/

	CPPUNIT_ASSERT ( data2 == data );
}

void testIterator ( size_t size ) {
	ext::vector < bool > ref = createTestVectorBool ( size );
	common::SparseBoolVector data ( ref );

	size_t count = 0;
	for ( bool boolean : ref ) {
		count += boolean;
	}

	for ( size_t onePosition : data ) {
		--count;
		CPPUNIT_ASSERT ( ref [ onePosition ] );
	}

	CPPUNIT_ASSERT ( count == 0 );
}

void SparseBoolVectorTest::testOnesIterator() {
	for ( size_t i = 0; i < 1000; i++ ) {
		testIterator ( i );
	}

	common::SparseBoolVector empty;
	CPPUNIT_ASSERT ( empty.begin ( ) == empty.end ( ) );
	empty.push_back ( false );
	CPPUNIT_ASSERT ( empty.begin ( ) == empty.end ( ) );
}

void SparseBoolVectorTest::testCompare() {
	common::SparseBoolVector first;
	common::SparseBoolVector second;
	common::SparseBoolVector third;
	common::SparseBoolVector fourth;
	common::SparseBoolVector fifth;
	common::SparseBoolVector sixth;

	first.push_back ( false );

	second.push_back ( true );

	third.push_back ( false );
	third.push_back ( false );

	fourth.push_back ( false );
	fourth.push_back ( true );

	fifth.push_back ( true );
	fifth.push_back ( false );

	sixth.push_back ( true );
	sixth.push_back ( true );

	ext::compare < common::SparseBoolVector > comp;
	CPPUNIT_ASSERT ( comp ( first, first ) == 0 );
	CPPUNIT_ASSERT ( comp ( first, second ) < 0 );

	CPPUNIT_ASSERT ( comp ( second, first ) > 0 );
	CPPUNIT_ASSERT ( comp ( second, second ) == 0 );

	CPPUNIT_ASSERT ( comp ( third, third ) == 0 );
	CPPUNIT_ASSERT ( comp ( third, fourth ) < 0 );
	CPPUNIT_ASSERT ( comp ( third, fifth ) < 0 );
	CPPUNIT_ASSERT ( comp ( third, sixth ) < 0 );

	CPPUNIT_ASSERT ( comp ( fourth, third ) > 0 );
	CPPUNIT_ASSERT ( comp ( fourth, fourth ) == 0 );
	CPPUNIT_ASSERT ( comp ( fourth, fifth ) < 0 );
	CPPUNIT_ASSERT ( comp ( fourth, sixth ) < 0 );

	CPPUNIT_ASSERT ( comp ( fifth, third ) > 0 );
	CPPUNIT_ASSERT ( comp ( fifth, fourth ) > 0 );
	CPPUNIT_ASSERT ( comp ( fifth, fifth ) == 0 );
	CPPUNIT_ASSERT ( comp ( fifth, sixth ) < 0 );

	CPPUNIT_ASSERT ( comp ( sixth, third ) > 0 );
	CPPUNIT_ASSERT ( comp ( sixth, fourth ) > 0 );
	CPPUNIT_ASSERT ( comp ( sixth, fifth ) > 0 );
	CPPUNIT_ASSERT ( comp ( sixth, sixth ) == 0 );
}

void SparseBoolVectorTest::testSubscript() {
	ext::vector < bool > ref = createTestVectorBool ( 2000 );
	common::SparseBoolVector data ( ref );

/*	std::cout << ref << std::endl;
	std::cout << data.data ( ) << std::endl;*/

	for ( size_t i = 0; i < ref.size ( ); ++ i )
		CPPUNIT_ASSERT_EQUAL_MESSAGE ( "Failed on index " + ext::to_string ( i ), ( bool ) ref [ i ], ( bool ) data [ i ] );
}

void SparseBoolVectorTest::testSubscript2() {
	ext::vector < bool > ref;
	common::SparseBoolVector data;

	ref.resize ( 2000 );
	data.resize ( 2000 );

	ref[520] = true;
	data[520] = true;

	CPPUNIT_ASSERT ( ( ext::vector < bool > ) data == ref );

	ref[521] = true;
	data[521] = true;

	CPPUNIT_ASSERT ( ( ext::vector < bool > ) data == ref );

	ref[128] = true;
	data[128] = true;

	CPPUNIT_ASSERT ( ( ext::vector < bool > ) data == ref );

	ref[255] = true;
	data[255] = true;

	CPPUNIT_ASSERT ( ( ext::vector < bool > ) data == ref );

	ref[256] = true;
	data[256] = true;

	CPPUNIT_ASSERT ( ( ext::vector < bool > ) data == ref );
}

void SparseBoolVectorTest::testOperatorAnd ( ) {
	ext::vector < bool > first;
	ext::vector < bool > second;
	ext::vector < bool > ref;

	first.resize ( 2000 );
	second.resize ( 2000 );
	ref.resize ( 2000 );

	first[520] = first[1] = first[3] = first[512] = first[511] = first[128] = first[127] = first[17] = first[53] = first[1999] = true;
	second[80] = second[1] = second[5] = second[51] = second[5] = second[7] = second[127] = second[1] = second[77] = second[999] = true;

	ref[1] = ref[127] = true;

	common::SparseBoolVector data;
	CPPUNIT_ASSERT ( ( first & second ) == ref );
}

void SparseBoolVectorTest::testXmlApi() {
	ext::vector < bool > ref = createTestVectorBool ( 2000 );
	common::SparseBoolVector data ( ref );

	ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens( data );
	std::string tmp;
	sax::SaxComposeInterface::printMemory(tmp, tokens);
//	std::cout << tmp << std::endl;

	ext::deque<sax::Token> tokens2;
	sax::SaxParseInterface::parseMemory(tmp, tokens2);
	common::SparseBoolVector data2 = factory::XmlDataFactory::fromTokens ( std::move ( tokens2 ) );

/*	std::cout << data.data ( ) << std::endl;
	std::cout << data2.data ( ) << std::endl;*/
	CPPUNIT_ASSERT( data == data2 );
}

