/*
 * LR0ItemsLabel.cpp
 *
 *  Created on: 29. 4. 2016
 *      Author: Martin Kocicka
 */

#include "LR0ItemsLabel.h"
#include <sax/FromXMLParserHelper.h>
#include <label/Label.h>
#include <object/Object.h>
#include <label/UniqueLabel.h>
#include <container/ObjectsMap.h>
#include <container/ObjectsSet.h>
#include <container/ObjectsVector.h>
#include <primitive/Unsigned.h>

#include <container/xml/ObjectsMap.h>
#include <container/xml/ObjectsSet.h>
#include <container/xml/ObjectsVector.h>
#include <primitive/xml/Unsigned.h>

#include <registration/ValuePrinterRegistration.hpp>
#include <registration/XmlRegistration.hpp>

namespace label {

LR0ItemsLabel::LR0ItemsLabel ( grammar::parsing::LR0Items items ) : m_items ( items ) {
}

LR0ItemsLabel::LR0ItemsLabel ( ) {
}

LabelBase* LR0ItemsLabel::clone ( ) const {
	return new LR0ItemsLabel ( *this );
}

LabelBase* LR0ItemsLabel::plunder ( ) && {
	return new LR0ItemsLabel ( std::move ( *this ) );
}

bool LR0ItemsLabel::operator < ( const LR0ItemsLabel & other ) const {
	return this->compare(other) < 0;
}

int LR0ItemsLabel::compare ( const LR0ItemsLabel & other ) const {
	if ( m_items < other.m_items ) {
		return -1;
	} else if ( m_items > other.m_items ) {
		return 1;
	} else {
		return 0;
	}
}

void LR0ItemsLabel::operator>>(std::ostream & out) const {
	out << "(LR0ItemsLabel " << m_items << ")";
}

LR0ItemsLabel::operator std::string ( ) const {
	std::stringstream ss;
	ss << *this;
	return ss.str ( );
}

LR0ItemsLabel LR0ItemsLabel::parse ( ext::deque<sax::Token>::iterator& input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, LR0ItemsLabel::getXmlTagName() );
	LR0ItemsLabel data ( core::xmlApi < grammar::parsing::LR0Items > ::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, LR0ItemsLabel::getXmlTagName() );
	return data;
}

void LR0ItemsLabel::compose(ext::deque<sax::Token>& out, const LR0ItemsLabel & label ) {
	out.emplace_back ( LR0ItemsLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT );
	core::xmlApi < grammar::parsing::LR0Items > ::compose ( out, label.m_items );
	out.emplace_back ( LR0ItemsLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT );
}

LabelBase* LR0ItemsLabel::inc ( ) && {
	return new UniqueLabel ( Label ( std::move ( *this ) ), primitive::Integer ( 0 ) );
}

} /* namespace label */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < label::LR0ItemsLabel > ( );

static auto xmlWrite = registration::XmlWriterRegister < label::LR0ItemsLabel > ( );
static auto xmlRead = registration::XmlReaderRegister < label::LR0ItemsLabel > ( );

static auto xmlGroup1 = registration::XmlRegisterTypeInGroup < label::Label, label::LR0ItemsLabel > ( );
static auto xmlGroup2 = registration::XmlRegisterTypeInGroup < object::Object, label::LR0ItemsLabel > ( );

} /* namespace */
