#ifndef VARIANT_TEST_H_
#define VARIANT_TEST_H_

#include <cppunit/extensions/HelperMacros.h>
#include <alib/compare>

class VariantTest : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE( VariantTest );
  CPPUNIT_TEST( testVariant );
  CPPUNIT_TEST( testVariantSet );
  CPPUNIT_TEST( testVariantSet2 );
  CPPUNIT_TEST( testVariantVoid );
  CPPUNIT_TEST( testVariantDefault );
  CPPUNIT_TEST( testVariantSameTypes );
  CPPUNIT_TEST( testSizeT );
  CPPUNIT_TEST( testMoveSemantics );
  CPPUNIT_TEST( testSetVariant );
  CPPUNIT_TEST_SUITE_END();

public:
struct test {
	int * holder;

	test() {
		//std::cout << "test()" << std::endl;
		holder = new int();
	}

	test(test&& old) : holder(nullptr) {
		//std::cout << "test(test&&)" << std::endl;
		std::swap(holder,old.holder);
	}

	test(const test& old) {
		  //std::cout << "test(const test&)" << std::endl;
		  holder = new int(*old.holder);
	}

	~test()
	{
		//std::cout << "~test()" << std::endl;
		delete holder;
	}

	bool operator<(const test& other) const {
		return *(this->holder) < *(other.holder);
	}

	bool operator==(const test& other) const {
		return *(this->holder) == *(other.holder);
	}

	int compare(const test& other) const {
		return *(this->holder) - *(other.holder);
	}

	friend std::ostream& operator<<(std::ostream& out, const test& other) {
		out << *(other.holder);
		return out;
	}
};

struct test2 {
	int m_i;

	test2(int i) : m_i(i) {}
};

public:
  void setUp();
  void tearDown();

  void testVariant();
  void testVariantSet();
  void testVariantSet2();
  void testVariantVoid();
  void testVariantDefault();
  void testVariantSameTypes();
  void testSizeT();
  void testMoveSemantics();
  void testSetVariant ( );
};

namespace ext {

template<>
struct compare<VariantTest::test> {
	int operator()(const VariantTest::test& first, const VariantTest::test& second) {
		return first.compare(second);
	}
};

} /* namespace ext */

#endif  // VARIANT_TEST_H_
