/*
 * XmlRegistry.cpp
 *
 *  Created on: 10. 7. 2017
 *	  Author: Jan Travnicek
 */

#include <registry/XmlRegistry.h>

#include <registry/XmlComposerRegistry.hpp>
#include <registry/XmlParserRegistry.hpp>
#include <registry/XmlContainerParserRegistry.hpp>

#include <abstraction/ImmediateValueAbstraction.hpp>
#include <abstraction/XmlTokensComposerAbstraction.hpp>
#include <abstraction/XmlTokensParserAbstraction.hpp>

namespace abstraction {

ext::set < std::string > XmlRegistry::listDataTypes ( ) {
	return XmlComposerRegistry::list ( );
}

ext::set < std::string > XmlRegistry::listDataTypeGroup ( const std::string & group ) {
	return XmlComposerRegistry::listGroup ( group );
}

std::shared_ptr < abstraction::OperationAbstraction > XmlRegistry::getXmlComposerAbstraction ( const std::string & param ) {
	return XmlComposerRegistry::getAbstraction ( param );
}

std::shared_ptr < abstraction::OperationAbstraction > XmlRegistry::getXmlParserAbstraction ( const std::string & typeName ) {
	return XmlParserRegistry::getAbstraction ( typeName );
}

std::shared_ptr < abstraction::OperationAbstraction > XmlRegistry::getXmlContainerParserAbstraction ( const std::string & container, const std::string & type ) {
	return XmlContainerParserRegistry::getAbstraction ( container, type );
}

std::shared_ptr < abstraction::UnaryOperationAbstraction < ext::deque < sax::Token >, const std::string & > > XmlRegistry::fileToTokensAbstraction ( const std::string & filename ) {
	std::shared_ptr < abstraction::ImmediateValueAbstraction < std::string > > file = std::make_shared < abstraction::ImmediateValueAbstraction < std::string > > ( filename );

	std::shared_ptr < abstraction::UnaryOperationAbstraction < ext::deque < sax::Token >, const std::string & > > res = std::make_shared < abstraction::XmlTokensParserAbstraction > ( );
	std::shared_ptr < abstraction::OperationAbstraction > tmp = res;

	tmp->attachInput ( file, 0, false );

	return res;
}

std::shared_ptr < abstraction::BinaryOperationAbstraction < void, const ext::deque < sax::Token > &, const std::string & > > XmlRegistry::tokensToFileAbstraction ( const std::string & filename ) {
	std::shared_ptr < abstraction::ImmediateValueAbstraction < std::string > > file = std::make_shared < abstraction::ImmediateValueAbstraction < std::string > > ( filename );

	std::shared_ptr < abstraction::BinaryOperationAbstraction < void, const ext::deque < sax::Token > &, const std::string & > > res = std::make_shared < abstraction::XmlTokensComposerAbstraction > ( );
	std::shared_ptr < abstraction::OperationAbstraction > tmp = res;

	tmp->attachInput ( file, 1, false );

	return res;

}
} /* namespace abstraction */
